/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.layout;

import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_CONNECTOR_SIZE;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_ASPECT_RATIO;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_HEIGHT;
import static org.fortiss.tooling.base.layout.DefaultLayoutConstants.DEFAULT_SHAPE_MINIMUM_WIDTH;
import static org.fortiss.tooling.base.ui.utils.EllipseLayoutUtils.calculateEllipseBorderPoint;
import static org.fortiss.tooling.base.ui.utils.EllipseLayoutUtils.clampEllipseShapeSize;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.getConnectorAngleAsDouble;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.getNodeSize;
import static org.fortiss.tooling.base.ui.utils.LayoutDataUtils.setConnectorAngle;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.editpart.ElementEditPartBase;
import org.fortiss.tooling.base.ui.editpart.StickyConnectorEditPartBase;
import org.fortiss.tooling.base.ui.utils.EllipseLayoutUtils.Polar;

/**
 * This layout configuration implements the ellipse and curve flavor. The
 * diagram has no grid. Its elements and free connectors can be placed freely.
 * Elements use an ellipse shape and sticky connectors are aligned with the
 * ellipse border. Connections use a bezier curve router with bend points being
 * achors to the curve.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 7DAAED45C777EA5D81BB846B82ECB9E7
 */
public class EllipseCurveLayoutDiagramConfiguration extends DiagramLayoutConfigurationBase {

	/** Shared instance. */
	public static final EllipseCurveLayoutDiagramConfiguration INSTANCE =
			new EllipseCurveLayoutDiagramConfiguration();

	/** Half size of default connector. */
	private static final int HALF_DEFAULT_CONNECTOR_SIZE = DEFAULT_CONNECTOR_SIZE / 2;

	/** The minimal size. */
	private static final Dimension MINIMUM_SIZE = new Dimension(DEFAULT_SHAPE_MINIMUM_WIDTH,
			DEFAULT_SHAPE_MINIMUM_HEIGHT);

	/** {@inheritDoc} */
	@Override
	public Dimension getMinimalElementSize() {
		return MINIMUM_SIZE;
	}

	/** {@inheritDoc} */
	@Override
	public void performStickyConnectorMove(
			StickyConnectorEditPartBase<? extends ILayoutedModelElement> connectorEditPart,
			ChangeBoundsRequest request) {
		ILayoutedModelElement connector = connectorEditPart.getModel();
		double angle = getConnectorAngleAsDouble(connector);

		Dimension stateSize = new Dimension(10, 10);
		if(connector.eContainer() instanceof ILayoutedModelElement) {
			org.fortiss.tooling.base.model.layout.Dimension parentSize =
					getNodeSize((ILayoutedModelElement)connector.eContainer());
			stateSize.setWidth(parentSize.getWidth());
			stateSize.setHeight(parentSize.getHeight());
		}
		Point oldPosition =
				calculateEllipseBorderPoint(angle, stateSize, HALF_DEFAULT_CONNECTOR_SIZE);

		Point newPosition =
				oldPosition.getTranslated(request.getMoveDelta()).translate(
						HALF_DEFAULT_CONNECTOR_SIZE, HALF_DEFAULT_CONNECTOR_SIZE);
		Dimension ellipseSize = connectorEditPart.getParentEditPart().getFigure().getSize();
		newPosition.translate(ellipseSize.getScaled(-.5));

		setConnectorAngle(connector, new Polar(newPosition).angle);
	}

	/** {@inheritDoc} */
	@Override
	public void alterStickyConnectorMoveFeedbackRequest(ChangeBoundsRequest request,
			StickyConnectorEditPartBase<? extends ILayoutedModelElement> connectorEditPart,
			IFigure connectorFigure) {
		ILayoutedModelElement stickyConnectorModel = connectorEditPart.getModel();
		double angle = getConnectorAngleAsDouble(stickyConnectorModel);

		// compute old bounds
		Dimension ellipseSize = connectorEditPart.getParentEditPart().getFigure().getSize();
		Point oldPosition =
				calculateEllipseBorderPoint(angle, ellipseSize, HALF_DEFAULT_CONNECTOR_SIZE)
						.translate(-HALF_DEFAULT_CONNECTOR_SIZE, -HALF_DEFAULT_CONNECTOR_SIZE);
		// apply movement delta
		Point newPosition = oldPosition.getTranslated(request.getMoveDelta());

		// fix new position to ellipse border
		newPosition.translate(ellipseSize.getScaled(-.5));
		newPosition.translate(HALF_DEFAULT_CONNECTOR_SIZE, HALF_DEFAULT_CONNECTOR_SIZE);
		// this converts from bounds to polar and back to ellipse border
		// location
		newPosition =
				calculateEllipseBorderPoint(new Polar(newPosition).angle, ellipseSize,
						HALF_DEFAULT_CONNECTOR_SIZE).translate(-HALF_DEFAULT_CONNECTOR_SIZE,
						-HALF_DEFAULT_CONNECTOR_SIZE);
		// update move delta with new position
		request.setMoveDelta(new Point().translate(newPosition.getDifference(oldPosition)));
	}

	/** {@inheritDoc} */
	@Override
	public void alterElementMoveAndResizeFeedbackRequest(ChangeBoundsRequest request,
			ElementEditPartBase<? extends ILayoutedModelElement> elementEditPart) {
		Dimension oldSizeDelta = request.getSizeDelta();
		Rectangle elementBounds = elementEditPart.getFigure().getBounds();
		Dimension oldSize =
				new Dimension(elementBounds.width + oldSizeDelta.width, elementBounds.height +
						oldSizeDelta.height);
		Dimension minimal = getMinimalElementSize();
		Dimension newSize =
				clampEllipseShapeSize(oldSize, request.getResizeDirection(), minimal.width,
						minimal.height, getAspectRatio());
		Dimension newSizeDelta =
				new Dimension(newSize.width - elementBounds.width, newSize.height -
						elementBounds.height);

		request.setSizeDelta(newSizeDelta);
	}

	/** Returns the aspect ratio to be used for ellipse resizing. */
	public double getAspectRatio() {
		return DEFAULT_SHAPE_ASPECT_RATIO;
	}
}
