/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.graphicsGL.util;

import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.color;

import java.util.List;

import org.fortiss.tooling.graphicsGL.model.Color3D;
import org.fortiss.tooling.graphicsGL.model.mesh.Face3D;
import org.fortiss.tooling.graphicsGL.model.mesh.Material3D;
import org.fortiss.tooling.graphicsGL.model.mesh.MaterialGroup3D;
import org.fortiss.tooling.graphicsGL.model.mesh.Mesh3D;
import org.fortiss.tooling.graphicsGL.model.mesh.MeshFactory;
import org.fortiss.tooling.graphicsGL.model.mesh.MeshPackage;
import org.fortiss.tooling.graphicsGL.model.mesh.Normal3D;
import org.fortiss.tooling.graphicsGL.model.mesh.Vertex3D;

/**
 * Model element factory for {@link MeshPackage}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 43BD82BAD0526AB085B925D9BC9ABFD3
 */
public class Mesh3DModelElementFactory {

	/** Creates a vertex. */
	public static Vertex3D vertex(float x, float y, float z) {
		Vertex3D v = MeshFactory.eINSTANCE.createVertex3D();
		v.setX(x);
		v.setY(y);
		v.setZ(z);
		return v;
	}

	/** Creates a normal. */
	public static Normal3D normal(float x, float y, float z) {
		Normal3D n = MeshFactory.eINSTANCE.createNormal3D();
		n.setX(x);
		n.setY(y);
		n.setZ(z);
		return n;
	}

	/** Creates a triangular face without normals. */
	public static Face3D triangleFace(Vertex3D v1, Vertex3D v2, Vertex3D v3) {
		Face3D face = MeshFactory.eINSTANCE.createFace3D();
		face.getVerticesList().add(vertex(v1.getX(), v1.getY(), v1.getZ()));
		face.getVerticesList().add(vertex(v2.getX(), v2.getY(), v2.getZ()));
		face.getVerticesList().add(vertex(v3.getX(), v3.getY(), v3.getZ()));
		return face;
	}

	/** Creates a triangular face with normals. */
	public static Face3D triangleFace(Vertex3D v1, Vertex3D v2, Vertex3D v3, Normal3D n1,
			Normal3D n2, Normal3D n3) {
		Face3D f = triangleFace(v1, v2, v3);
		f.getNormalsList().add(normal(n1.getX(), n1.getY(), n1.getZ()));
		f.getNormalsList().add(normal(n2.getX(), n2.getY(), n2.getZ()));
		f.getNormalsList().add(normal(n3.getX(), n3.getY(), n3.getZ()));
		return f;
	}

	/** Creates a quad face without normals. */
	public static Face3D quadFace(Vertex3D v1, Vertex3D v2, Vertex3D v3, Vertex3D v4) {
		Face3D face = MeshFactory.eINSTANCE.createFace3D();
		face.getVerticesList().add(vertex(v1.getX(), v1.getY(), v1.getZ()));
		face.getVerticesList().add(vertex(v2.getX(), v2.getY(), v2.getZ()));
		face.getVerticesList().add(vertex(v3.getX(), v3.getY(), v3.getZ()));
		face.getVerticesList().add(vertex(v4.getX(), v4.getY(), v4.getZ()));
		return face;
	}

	/** Creates a quad face without normals. */
	public static Face3D quadFace(Vertex3D v1, Vertex3D v2, Vertex3D v3, Vertex3D v4, Normal3D n1,
			Normal3D n2, Normal3D n3, Normal3D n4) {
		Face3D f = quadFace(v1, v2, v3, v4);
		f.getNormalsList().add(normal(n1.getX(), n1.getY(), n1.getZ()));
		f.getNormalsList().add(normal(n2.getX(), n2.getY(), n2.getZ()));
		f.getNormalsList().add(normal(n3.getX(), n3.getY(), n3.getZ()));
		f.getNormalsList().add(normal(n4.getX(), n4.getY(), n4.getZ()));
		return f;
	}

	/** Creates a mesh with the given vertices and faces. */
	public static Mesh3D mesh(List<MaterialGroup3D> groups) {
		Mesh3D m = MeshFactory.eINSTANCE.createMesh3D();
		m.getMaterialGroupsList().addAll(groups);
		return m;
	}

	/** The default material with 0.4 ambient, 0.8 diffuse and 1.0 specular property. */
	public static final Material3D DEFAULT_MATERIAL = material(color(0.4f, 0.4f, 0.4f),
			color(0.8f, 0.8f, 0.8f), color(1.0f, 1.0f, 1.0f), 1.0f);

	/** Creates a material group. */
	public static MaterialGroup3D materialGroup(Material3D m, List<Face3D> faces,
			boolean smoothShading) {
		MaterialGroup3D group = MeshFactory.eINSTANCE.createMaterialGroup3D();
		group.setMaterial(material(m.getAmbient(), m.getDiffuse(), m.getSpecular(),
				m.getSpecularCoeff()));
		group.setSmoothShading(smoothShading);
		group.getFacesList().addAll(faces);
		return group;
	}

	/** Creates a material with ambient color only. */
	public static Material3D material(Color3D ambient) {
		Material3D m = MeshFactory.eINSTANCE.createMaterial3D();
		m.setAmbient(color(ambient.getRed(), ambient.getGreen(), ambient.getBlue()));
		return m;
	}

	/** Creates a material with ambient and diffuse color. */
	public static Material3D material(Color3D ambient, Color3D diffuse) {
		Material3D m = material(ambient);
		m.setDiffuse(color(diffuse.getRed(), diffuse.getGreen(), diffuse.getBlue()));
		return m;
	}

	/** Creates a material with ambient, diffuse, and specular color. */
	public static Material3D material(Color3D ambient, Color3D diffuse, Color3D specular,
			float specularCoeff) {
		Material3D m = material(ambient, diffuse);
		m.setSpecular(color(specular.getRed(), specular.getGreen(), specular.getBlue()));
		m.setSpecularCoeff(specularCoeff);
		return m;
	}
}
