/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.getBundle;
import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.getConfigurationElements;
import static org.fortiss.tooling.kernel.utils.ExtensionPointUtils.loadClass;
import static org.fortiss.tooling.kernel.utils.LoggingUtils.error;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map.Entry;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.conqat.lib.commons.collections.UnmodifiableList;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.extension.IPrototypeProvider;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.extension.data.ProtoypeCategory;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.IPrototypeService;
import org.osgi.framework.Bundle;

/**
 * This class implements the {@link IPrototypeService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 7348244C108974051B1E151F2B83BD61
 */
public class PrototypeService implements IPrototypeService {

	/** The prototype provider extension point ID. */
	private static final String EXTENSION_POINT_NAME =
			"org.fortiss.tooling.kernel.modelPrototypeProvider";

	/** The prototype provider configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "modelPrototypeProvider";

	/** Stores the registered prototype providers. */
	private final List<IPrototypeProvider> prototypeProviderList =
			new ArrayList<IPrototypeProvider>();

	/** Stores the optional category affiliation for the prototype. */
	private final HashMap<Class<? extends EObject>, String> map =
			new HashMap<Class<? extends EObject>, String>();

	/** {@inheritDoc} */
	@Override
	public String getCategory(EObject element) {
		return map.get(element.getClass());
	}

	/** Constructor. */
	public PrototypeService() {
		setupPrototypes();
	}

	/** Initializes the prototype list from plugin extensions. */
	private void setupPrototypes() {
		for(IConfigurationElement ce : getConfigurationElements(EXTENSION_POINT_NAME,
				CONFIGURATION_ELEMENT_NAME)) {
			Bundle bundle = getBundle(ce);
			try {
				Class<?> handlerClass = loadClass(ce.getAttribute("provider"), bundle);
				IPrototypeProvider provider =
						(IPrototypeProvider)handlerClass.getConstructor().newInstance();
				prototypeProviderList.add(provider);
			} catch(Exception ex) {
				error(ToolingKernelActivator.getDefault(), ex.getMessage(), ex);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public UnmodifiableList<IPrototypeProvider> getPrototypeProviders() {
		return CollectionUtils.asUnmodifiable(prototypeProviderList);
	}

	/** {@inheritDoc} */
	@Override
	public UnmodifiableList<Prototype> getAllPrototypes() {
		return getPrototypes(false);
	}

	/** {@inheritDoc} */
	@Override
	public UnmodifiableList<Prototype> getPrimaryPrototypes() {
		return getPrototypes(true);
	}

	/** Returns prototypes including or excluding non-primaries. */
	private UnmodifiableList<Prototype> getPrototypes(boolean primaryOnly) {
		List<Prototype> result = new ArrayList<Prototype>();
		for(IPrototypeProvider provider : prototypeProviderList) {
			for(Prototype p : provider.getPrototypes()) {
				if(!primaryOnly || p.isPrimary()) {
					result.add(p);
				}
			}
		}
		return CollectionUtils.asUnmodifiable(result);
	}

	/** {@inheritDoc} */
	@Override
	public List<Prototype> getComposablePrototypes(Class<? extends EObject> modelElementType) {
		List<Prototype> result = new LinkedList<Prototype>();
		for(Prototype prototype : getAllPrototypes()) {
			if(IElementCompositorService.INSTANCE.canComposePrototype(modelElementType, prototype)) {
				result.add(prototype);
			}
		}
		return result;
	}

	/** {@inheritDoc} */
	@Override
	public List<Object> getAllPrototypesByCategory() {
		init();
		ArrayList<Object> result = new ArrayList<Object>();
		ArrayList<Object> uncategorized = new ArrayList<Object>();
		HashMap<String, ProtoypeCategory> categories = new HashMap<String, ProtoypeCategory>();

		for(Prototype p : getAllPrototypes()) {
			String categoryName = map.get(p.getPrototype().getClass());
			if(categoryName != null) {
				if(!categories.containsKey(categoryName)) {
					categories.put(categoryName, new ProtoypeCategory(categoryName));
				}
				categories.get(categoryName).add(p);
			} else {
				uncategorized.add(p);
			}
		}
		for(ProtoypeCategory pc : categories.values()) {
			result.add(pc);
		}
		result.addAll(uncategorized);
		return result;
	}

	/** Initially fills the map with categorized elements. */
	private void init() {
		if(!map.isEmpty()) {
			return;
		}
		for(IPrototypeProvider provider : prototypeProviderList) {
			for(Entry<Class<? extends EObject>, String> entry : provider.getCategories().entrySet()) {
				map.put(entry.getKey(), entry.getValue());
			}
		}
	}
}
