/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.ITransformationContext;
import org.fortiss.tooling.kernel.extension.data.TransformationProviderChain;
import org.fortiss.tooling.kernel.extension.exception.ChainTransformationFailedException;
import org.fortiss.tooling.kernel.service.ITransformationService;

/**
 * Utility methods for the {@link ITransformationService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 4540F72C38F7E1C6A441AD7A0AA30E75
 */
public final class TransformationUtils {

	/**
	 * Searches a transformation chain for the given model element and the
	 * target class and returns the transformation result.
	 * 
	 * @param sourceElement
	 *            the source of the transformation
	 * @param targetClass
	 *            the class of the transformation result
	 * @param context
	 *            the transformation context
	 * @return the transformation result (an instance of class <code>targetClass</code>).
	 * @throws ChainTransformationFailedException
	 *             if no transformation chain could be found or none of the
	 *             possible chains succeeded.
	 */
	@SuppressWarnings("unchecked")
	public static <T extends Object> T createTransformedObjectFor(final EObject sourceElement,
			final Class<T> targetClass, ITransformationContext context)
			throws ChainTransformationFailedException {
		List<TransformationProviderChain> chainList =
				ITransformationService.INSTANCE.getTransformationProviderChain(
						sourceElement.getClass(), targetClass, context);
		ChainTransformationFailedException e = null;
		if(!chainList.isEmpty()) {
			Object executableObject = null;
			for(TransformationProviderChain chain : chainList) {
				try {
					// for the moment, we use the first chain
					// alternatives need an interpretation of the context, which
					// we do not use at the moment
					executableObject = chain.transform(sourceElement, context);
					e = null;
					break;
				} catch(ChainTransformationFailedException ctfe) {
					// fall through
					e = ctfe;
				}
			}
			if(e == null && executableObject != null) {
				return (T)executableObject;
			}
		}
		if(e != null) {
			throw e;
		}
		throw new ChainTransformationFailedException(null, null, null, null) {
			/** {@inheritDoc} */
			@Override
			public String getMessage() {
				return "No transformation chain found from " + sourceElement + " to " +
						targetClass.getName();
			}
		};
	}

	/**
	 * Searches a transformation chain for the given model element and the
	 * target class and returns the transformation result. This method returns <code>null</code> if
	 * the transformation failed. The exception is
	 * suppressed.
	 * 
	 * This is a fail-silent version of
	 * {@link #createTransformedObjectFor(EObject, Class, ITransformationContext)} .
	 * 
	 * @param sourceElement
	 *            the source of the transformation
	 * @param targetClass
	 *            the class of the transformation result
	 * @param context
	 *            the transformation context
	 * @return the transformation result (an instance of class <code>targetClass</code>).
	 */
	public static <T extends Object> T createTransformedObjectWithoutExceptionFor(
			EObject sourceElement, Class<T> targetClass, ITransformationContext context) {
		try {
			T result = createTransformedObjectFor(sourceElement, targetClass, context);
			return result;
		} catch(ChainTransformationFailedException ctfe) {
			return null;
		}
	}

	/**
	 * Searches a transformation chain for the given model element and the target class and returns
	 * {@code true} if such a chain exists.
	 * 
	 * @param sourceElement
	 *            the source of the transformation
	 * @param targetClass
	 *            the class of the transformation result
	 * @param context
	 *            the transformation context
	 * @return {@code true} if a transformation chain exists, {@code false} otherwise
	 */
	public static boolean canTransform(EObject sourceElement, Class<?> targetClass,
			ITransformationContext context) {
		return ITransformationService.INSTANCE.canTransform(sourceElement, targetClass, context);
	}
}
