/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.model.IIdLabeled;

/**
 * Utility methods for unique IDs of {@link IIdLabeled} model elements.
 * IMPORTANT: Use this methods with care! It is usually the task of the persistency
 * layer to assign consistent IDs. However, test models and programmatic models
 * may need to use this capability without the persistency mechanism in place.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: A5E21261E29DDDE24057C442595C269B
 */
public class UniqueIDUtils {

	/**
	 * Generates missing IDs of the given model starting from the given current
	 * maximum ID.
	 * 
	 * @param existingModel
	 *            the model to be updated
	 * @param currentMaxId
	 *            the current maximum ID
	 */
	public static int generateMissingIDs(EObject existingModel, int currentMaxId) {
		for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				if(((IIdLabeled)eo).getId() <= 0) {
					((IIdLabeled)eo).setId((++currentMaxId));
				}
			}
		}
		return currentMaxId;
	}

	/**
	 * Returns the next ID not used within the given model
	 * 
	 * @param existingModel
	 *            the model to be searched
	 */
	public static int getLargestID(EObject existingModel) {
		int max = 0;
		for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				if(((IIdLabeled)eo).getId() > max) {
					max = ((IIdLabeled)eo).getId();
				}
			}
		}
		return max;
	}

	/**
	 * Generates all IDs of the given model. Already existing IDs are
	 * overwritten.
	 * 
	 * @param existingModel
	 *            the model to be updated
	 */
	public static void generateAllIDs(EObject existingModel) {
		int currentId = 0;
		for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				((IIdLabeled)eo).setId((++currentId));
			}
		}
	}

	/**
	 * Assigns unique IDs to the given object and its content.
	 * 
	 * @param object
	 *            the model element to be prepared
	 * @param existingModel
	 *            the existing model to be considered for used IDs if object
	 *            already contains IDs > 0
	 * @param currentMaxId
	 *            the current maximum ID to be started
	 * @return the new current maximum ID
	 */
	public static int prepareIDs(EObject object, EObject existingModel, int currentMaxId) {
		if(object == null) {
			return currentMaxId;
		}

		boolean needSmart = false;
		if(object instanceof IIdLabeled) {
			if(((IIdLabeled)object).getId() <= 0) {
				((IIdLabeled)object).setId(++currentMaxId);
			} else {
				needSmart = true;
			}
		}

		for(Iterator<EObject> i = object.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				if(((IIdLabeled)eo).getId() <= 0) {
					((IIdLabeled)eo).setId(++currentMaxId);
				} else {
					needSmart = true;
				}
			}
		}

		if(needSmart) {
			Set<Integer> usedIDs = new HashSet<Integer>();
			for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
				EObject eo = i.next();
				if(eo instanceof IIdLabeled) {
					usedIDs.add(((IIdLabeled)eo).getId());
				}
			}
			if(object instanceof IIdLabeled) {

				if(!usedIDs.add(((IIdLabeled)object).getId())) {
					int newId = ++currentMaxId;
					((IIdLabeled)object).setId(newId);
					usedIDs.add(newId);
				}
			}

			for(Iterator<EObject> i = object.eAllContents(); i.hasNext();) {
				EObject eo = i.next();
				if(eo instanceof IIdLabeled) {
					if(!usedIDs.add(((IIdLabeled)eo).getId())) {
						int newId = ++currentMaxId;
						((IIdLabeled)eo).setId(newId);
						usedIDs.add(newId);
					}
				}
			}
		}
		return currentMaxId;
	}
}
